﻿using UnityEngine;

namespace Cue.Sys.Vam
{
	class ColliderBodyPart : VamBodyPart
	{
		private Collider[] colliders_;
		private FreeControllerV3 fc_;
		private Rigidbody rb2_, closestRb_;

		public ColliderBodyPart(
			VamAtom a, int type, Collider[] cs, FreeControllerV3 fc,
			Rigidbody rb, Rigidbody closestRb)
				: base(a, type)
		{
			colliders_ = cs;
			fc_ = fc;
			rb2_ = rb;
			closestRb_ = closestRb;
		}

		public override Rigidbody Rigidbody
		{
			get { return rb2_ ?? closestRb_; }
		}

		public override FreeControllerV3 Controller
		{
			get { return fc_; }
		}

		public override bool CanGrab
		{
			get { return (fc_ != null); }
		}

		public override bool Grabbed
		{
			get { return (fc_?.isGrabbing ?? false); }
		}

		private Collider MainCollider
		{
			get { return colliders_[0]; }
		}

		public override Vector3 ControlPosition
		{
			get { return U.FromUnity(MainCollider.bounds.center); }
			set { Log.Error("cannot move colliders"); }
		}

		public override Quaternion ControlRotation
		{
			get { return U.FromUnity(MainCollider.transform.rotation); }
			set { Log.Error("cannot rotate colliders"); }
		}

		public override Vector3 Position
		{
			get { return ControlPosition; }
		}

		public override Quaternion Rotation
		{
			get { return ControlRotation; }
		}

		public override bool ContainsTransform(Transform t, bool debug)
		{
			if (rb2_ == null)
			{
				if (debug)
					Log.Error($"no rb, checking {colliders_.Length} colliders");

				for (int i = 0; i < colliders_.Length; ++i)
				{
					if (colliders_[i].transform == t)
					{
						if (debug)
							Log.Error($"{t.name} is collider #{i}");

						return true;
					}
					else
					{
						if (debug)
							Log.Error($"not collider {colliders_[i].transform.name}");
					}
				}
			}
			else
			{
				if (rb2_.transform == t)
				{
					if (debug)
						Log.Error($"has rb, found {t.name}");

					return true;
				}
				else
				{
					if (debug)
						Log.Error($"has rb, {t.name} is not {rb2_.name}");
				}
			}

			return false;
		}

		protected override Collider[] GetColliders()
		{
			return colliders_;
		}

		public override string ToString()
		{
			var ignore = new string[]
			{
				"AutoColliderFemaleAutoColliders",
				"AutoColliderMaleAutoColliders"
			};

			string s = "";

			if (rb2_ == null)
			{
				foreach (var i in ignore)
				{
					if (MainCollider.name.StartsWith(i))
					{
						s = MainCollider.name.Substring(i.Length);
						break;
					}
				}

				if (s == "")
					s = MainCollider.name;
			}
			else
			{
				s += rb2_.name;
			}

			if (fc_ != null)
				s = fc_.name + "." + s;

			return "collider " + s;
		}
	}
}
